Groups | Search | Server Info | Keyboard shortcuts | Login | Register [http] [https] [nntp] [nntps]


Groups > linux.kernel > #1638290 > unrolled thread

[PATCH v2 7/8] drm: Use mode_valid() in atomic modeset

Started byJose Abreu <Jose.Abreu@synopsys.com>
First post2017-05-09 19:10 +0200
Last post2017-05-10 20:00 +0200
Articles 3 — 3 participants

Back to article view | Back to linux.kernel

This discussion starts older than the indexed window; earlier articles aren't shown. The article labeled Started by below is the oldest one visible, not the original post.


Contents

  [PATCH v2 7/8] drm: Use mode_valid() in atomic modeset Jose Abreu <Jose.Abreu@synopsys.com> - 2017-05-09 19:10 +0200
    Re: [PATCH v2 7/8] drm: Use mode_valid() in atomic modeset Archit Taneja <architt@codeaurora.org> - 2017-05-10 18:10 +0200
      Re: [PATCH v2 7/8] drm: Use mode_valid() in atomic modeset Daniel Vetter <daniel@ffwll.ch> - 2017-05-10 20:00 +0200

#1638290 — [PATCH v2 7/8] drm: Use mode_valid() in atomic modeset

FromJose Abreu <Jose.Abreu@synopsys.com>
Date2017-05-09 19:10 +0200
Subject[PATCH v2 7/8] drm: Use mode_valid() in atomic modeset
Message-ID<tFgO5-4HS-3@gated-at.bofh.it>
This patches makes use of the new mode_valid() callbacks introduced
previously to validate the full video pipeline when modesetting.

This calls the connector->mode_valid(), encoder->mode_valid(),
bridge->mode_valid() and crtc->mode_valid() so that we can
make sure that the mode will be accepted in every components.

Signed-off-by: Jose Abreu <joabreu@synopsys.com>
Cc: Carlos Palminha <palminha@synopsys.com>
Cc: Alexey Brodkin <abrodkin@synopsys.com>
Cc: Ville Syrjälä <ville.syrjala@linux.intel.com>
Cc: Daniel Vetter <daniel.vetter@ffwll.ch>
Cc: Dave Airlie <airlied@linux.ie>
Cc: Andrzej Hajda <a.hajda@samsung.com>
Cc: Archit Taneja <architt@codeaurora.org>
---

Changes v1->v2:
	- Removed call to connector->mode_valid (Ville, Daniel)
	- Change function name (Ville)
	- Use for_each_new_connector_in_state (Ville)
	- Do not validate if connector and mode didn't change (Ville)
	- Use new helpers to call mode_valid

 drivers/gpu/drm/drm_atomic_helper.c | 75 +++++++++++++++++++++++++++++++++++--
 1 file changed, 72 insertions(+), 3 deletions(-)

diff --git a/drivers/gpu/drm/drm_atomic_helper.c b/drivers/gpu/drm/drm_atomic_helper.c
index 8be9719..19d0ea1 100644
--- a/drivers/gpu/drm/drm_atomic_helper.c
+++ b/drivers/gpu/drm/drm_atomic_helper.c
@@ -452,6 +452,69 @@ static int handle_conflicting_encoders(struct drm_atomic_state *state,
 	return 0;
 }
 
+static enum drm_mode_status mode_valid_path(struct drm_connector *connector,
+					    struct drm_encoder *encoder,
+					    struct drm_crtc *crtc,
+					    struct drm_display_mode *mode)
+{
+	enum drm_mode_status ret;
+
+	ret = drm_encoder_mode_valid(encoder, mode);
+	if (ret != MODE_OK) {
+		DRM_DEBUG_ATOMIC("[ENCODER:%d:%s] mode_valid() failed\n",
+				encoder->base.id, encoder->name);
+		return ret;
+	}
+
+	ret = drm_bridge_mode_valid(encoder->bridge, mode);
+	if (ret != MODE_OK) {
+		DRM_DEBUG_ATOMIC("[BRIDGE] mode_valid() failed\n");
+		return ret;
+	}
+
+	ret = drm_crtc_mode_valid(crtc, mode);
+	if (ret != MODE_OK) {
+		DRM_DEBUG_ATOMIC("[CRTC:%d:%s] mode_valid() failed\n",
+				crtc->base.id, crtc->name);
+		return ret;
+	}
+
+	return ret;
+}
+
+static int
+mode_valid(struct drm_atomic_state *state)
+{
+	struct drm_connector_state *conn_state;
+	struct drm_connector *connector;
+	int i;
+
+	for_each_new_connector_in_state(state, connector, conn_state, i) {
+		struct drm_encoder *encoder = conn_state->best_encoder;
+		struct drm_crtc *crtc = conn_state->crtc;
+		struct drm_crtc_state *crtc_state;
+		enum drm_mode_status mode_status;
+		struct drm_display_mode *mode;
+
+		if (!crtc || !encoder)
+			continue;
+
+		crtc_state = drm_atomic_get_new_crtc_state(state, crtc);
+		if (!crtc_state)
+			continue;
+		if (!crtc_state->mode_changed && !crtc_state->connectors_changed)
+			continue;
+
+		mode = &crtc_state->mode;
+
+		mode_status = mode_valid_path(connector, encoder, crtc, mode);
+		if (mode_status != MODE_OK)
+			return -EINVAL;
+	}
+
+	return 0;
+}
+
 /**
  * drm_atomic_helper_check_modeset - validate state object for modeset changes
  * @dev: DRM device
@@ -466,13 +529,15 @@ static int handle_conflicting_encoders(struct drm_atomic_state *state,
  * 2. &drm_connector_helper_funcs.atomic_check to validate the connector state.
  * 3. If it's determined a modeset is needed then all connectors on the affected crtc
  *    crtc are added and &drm_connector_helper_funcs.atomic_check is run on them.
  * 4. &drm_bridge_funcs.mode_fixup is called on all encoder bridges.
  * 5. &drm_encoder_helper_funcs.atomic_check is called to validate any encoder state.
  *    This function is only called when the encoder will be part of a configured crtc,
  *    it must not be used for implementing connector property validation.
  *    If this function is NULL, &drm_atomic_encoder_helper_funcs.mode_fixup is called
  *    instead.
- * 6. &drm_crtc_helper_funcs.mode_fixup is called last, to fix up the mode with crtc constraints.
+ * 7. &drm_crtc_helper_funcs.mode_fixup is called last, to fix up the mode with crtc constraints.
  *
  * &drm_crtc_state.mode_changed is set when the input mode is changed.
  * &drm_crtc_state.connectors_changed is set when a connector is added or
@@ -617,6 +682,10 @@ static int handle_conflicting_encoders(struct drm_atomic_state *state,
 			return ret;
 	}
 
+	ret = mode_valid(state);
+	if (ret)
+		return ret;
+
 	return mode_fixup(state);
 }
 EXPORT_SYMBOL(drm_atomic_helper_check_modeset);
@@ -466,13 +529,15 @@ static int handle_conflicting_encoders(struct drm_atomic_state *state,
  * 3. If it's determined a modeset is needed then all connectors on the affected crtc
  *    crtc are added and &drm_connector_helper_funcs.atomic_check is run on them.
- * 4. &drm_bridge_funcs.mode_fixup is called on all encoder bridges.
- * 5. &drm_encoder_helper_funcs.atomic_check is called to validate any encoder state.
+ * 4. &drm_encoder_helper_funcs.mode_valid, &drm_bridge_funcs.mode_valid and
+ *    &drm_crtc_helper_funcs.mode_valid are called on the affected components.
+ * 5. &drm_bridge_funcs.mode_fixup is called on all encoder bridges.
+ * 6. &drm_encoder_helper_funcs.atomic_check is called to validate any encoder state.
  *    This function is only called when the encoder will be part of a configured crtc,
  *    it must not be used for implementing connector property validation.
-- 
1.9.1

[toc] | [next] | [standalone]


#1638937

FromArchit Taneja <architt@codeaurora.org>
Date2017-05-10 18:10 +0200
Message-ID<tFClA-2QH-21@gated-at.bofh.it>
In reply to#1638290

On 5/9/2017 10:30 PM, Jose Abreu wrote:
> This patches makes use of the new mode_valid() callbacks introduced
> previously to validate the full video pipeline when modesetting.
> 
> This calls the connector->mode_valid(), encoder->mode_valid(),
> bridge->mode_valid() and crtc->mode_valid() so that we can
> make sure that the mode will be accepted in every components.
> 
> Signed-off-by: Jose Abreu <joabreu@synopsys.com>
> Cc: Carlos Palminha <palminha@synopsys.com>
> Cc: Alexey Brodkin <abrodkin@synopsys.com>
> Cc: Ville Syrjälä <ville.syrjala@linux.intel.com>
> Cc: Daniel Vetter <daniel.vetter@ffwll.ch>
> Cc: Dave Airlie <airlied@linux.ie>
> Cc: Andrzej Hajda <a.hajda@samsung.com>
> Cc: Archit Taneja <architt@codeaurora.org>
> ---
> 
> Changes v1->v2:
> 	- Removed call to connector->mode_valid (Ville, Daniel)
> 	- Change function name (Ville)
> 	- Use for_each_new_connector_in_state (Ville)
> 	- Do not validate if connector and mode didn't change (Ville)
> 	- Use new helpers to call mode_valid
> 
>   drivers/gpu/drm/drm_atomic_helper.c | 75 +++++++++++++++++++++++++++++++++++--
>   1 file changed, 72 insertions(+), 3 deletions(-)
> 
> diff --git a/drivers/gpu/drm/drm_atomic_helper.c b/drivers/gpu/drm/drm_atomic_helper.c
> index 8be9719..19d0ea1 100644
> --- a/drivers/gpu/drm/drm_atomic_helper.c
> +++ b/drivers/gpu/drm/drm_atomic_helper.c
> @@ -452,6 +452,69 @@ static int handle_conflicting_encoders(struct drm_atomic_state *state,
>   	return 0;
>   }
>   
> +static enum drm_mode_status mode_valid_path(struct drm_connector *connector,
> +					    struct drm_encoder *encoder,
> +					    struct drm_crtc *crtc,
> +					    struct drm_display_mode *mode)
> +{
> +	enum drm_mode_status ret;
> +
> +	ret = drm_encoder_mode_valid(encoder, mode);
> +	if (ret != MODE_OK) {
> +		DRM_DEBUG_ATOMIC("[ENCODER:%d:%s] mode_valid() failed\n",
> +				encoder->base.id, encoder->name);
> +		return ret;
> +	}
> +
> +	ret = drm_bridge_mode_valid(encoder->bridge, mode);
> +	if (ret != MODE_OK) {
> +		DRM_DEBUG_ATOMIC("[BRIDGE] mode_valid() failed\n");
> +		return ret;
> +	}
> +
> +	ret = drm_crtc_mode_valid(crtc, mode);
> +	if (ret != MODE_OK) {
> +		DRM_DEBUG_ATOMIC("[CRTC:%d:%s] mode_valid() failed\n",
> +				crtc->base.id, crtc->name);
> +		return ret;
> +	}
> +
> +	return ret;
> +}
> +
> +static int
> +mode_valid(struct drm_atomic_state *state)
> +{
> +	struct drm_connector_state *conn_state;
> +	struct drm_connector *connector;
> +	int i;
> +
> +	for_each_new_connector_in_state(state, connector, conn_state, i) {
> +		struct drm_encoder *encoder = conn_state->best_encoder;
> +		struct drm_crtc *crtc = conn_state->crtc;
> +		struct drm_crtc_state *crtc_state;
> +		enum drm_mode_status mode_status;
> +		struct drm_display_mode *mode;
> +
> +		if (!crtc || !encoder)
> +			continue;
> +
> +		crtc_state = drm_atomic_get_new_crtc_state(state, crtc);
> +		if (!crtc_state)
> +			continue;
> +		if (!crtc_state->mode_changed && !crtc_state->connectors_changed)
> +			continue;
> +
> +		mode = &crtc_state->mode;
> +
> +		mode_status = mode_valid_path(connector, encoder, crtc, mode);
> +		if (mode_status != MODE_OK)
> +			return -EINVAL;
> +	}
> +
> +	return 0;
> +}
> +
>   /**
>    * drm_atomic_helper_check_modeset - validate state object for modeset changes
>    * @dev: DRM device
> @@ -466,13 +529,15 @@ static int handle_conflicting_encoders(struct drm_atomic_state *state,
>    * 2. &drm_connector_helper_funcs.atomic_check to validate the connector state.
>    * 3. If it's determined a modeset is needed then all connectors on the affected crtc
>    *    crtc are added and &drm_connector_helper_funcs.atomic_check is run on them.
> - * 4. &drm_bridge_funcs.mode_fixup is called on all encoder bridges.
> - * 5. &drm_encoder_helper_funcs.atomic_check is called to validate any encoder state.
> + * 4. &drm_encoder_helper_funcs.mode_valid, &drm_bridge_funcs.mode_valid and
> + *    &drm_crtc_helper_funcs.mode_valid are called on the affected components.
> + * 5. &drm_bridge_funcs.mode_fixup is called on all encoder bridges.
> + * 6. &drm_encoder_helper_funcs.atomic_check is called to validate any encoder state.
>    *    This function is only called when the encoder will be part of a configured crtc,
>    *    it must not be used for implementing connector property validation.
>    *    If this function is NULL, &drm_atomic_encoder_helper_funcs.mode_fixup is called
>    *    instead.
> - * 6. &drm_crtc_helper_funcs.mode_fixup is called last, to fix up the mode with crtc constraints.
> + * 7. &drm_crtc_helper_funcs.mode_fixup is called last, to fix up the mode with crtc constraints.
>    *
>    * &drm_crtc_state.mode_changed is set when the input mode is changed.
>    * &drm_crtc_state.connectors_changed is set when a connector is added or
> @@ -617,6 +682,10 @@ static int handle_conflicting_encoders(struct drm_atomic_state *state,
>   			return ret;
>   	}
>   
> +	ret = mode_valid(state);
> +	if (ret)
> +		return ret;
> +

Since we've ensured that the modes won't fail, can mode_fixup() and
the mode_fixup() ops for crtc/bridge/encoders be assured to not fail
too? This is assuming that all drivers have moved to using the new
mode_valid() ops correctly.

Thanks,
Archit

>   	return mode_fixup(state);
>   }
>   EXPORT_SYMBOL(drm_atomic_helper_check_modeset);
> 

-- 
The Qualcomm Innovation Center, Inc. is a member of the Code Aurora Forum,
hosted by The Linux Foundation

[toc] | [prev] | [next] | [standalone]


#1638986

FromDaniel Vetter <daniel@ffwll.ch>
Date2017-05-10 20:00 +0200
Message-ID<tFE42-3Hp-9@gated-at.bofh.it>
In reply to#1638937
On Wed, May 10, 2017 at 09:38:00PM +0530, Archit Taneja wrote:
> 
> 
> On 5/9/2017 10:30 PM, Jose Abreu wrote:
> > This patches makes use of the new mode_valid() callbacks introduced
> > previously to validate the full video pipeline when modesetting.
> > 
> > This calls the connector->mode_valid(), encoder->mode_valid(),
> > bridge->mode_valid() and crtc->mode_valid() so that we can
> > make sure that the mode will be accepted in every components.
> > 
> > Signed-off-by: Jose Abreu <joabreu@synopsys.com>
> > Cc: Carlos Palminha <palminha@synopsys.com>
> > Cc: Alexey Brodkin <abrodkin@synopsys.com>
> > Cc: Ville Syrjälä <ville.syrjala@linux.intel.com>
> > Cc: Daniel Vetter <daniel.vetter@ffwll.ch>
> > Cc: Dave Airlie <airlied@linux.ie>
> > Cc: Andrzej Hajda <a.hajda@samsung.com>
> > Cc: Archit Taneja <architt@codeaurora.org>
> > ---
> > 
> > Changes v1->v2:
> > 	- Removed call to connector->mode_valid (Ville, Daniel)
> > 	- Change function name (Ville)
> > 	- Use for_each_new_connector_in_state (Ville)
> > 	- Do not validate if connector and mode didn't change (Ville)
> > 	- Use new helpers to call mode_valid
> > 
> >   drivers/gpu/drm/drm_atomic_helper.c | 75 +++++++++++++++++++++++++++++++++++--
> >   1 file changed, 72 insertions(+), 3 deletions(-)
> > 
> > diff --git a/drivers/gpu/drm/drm_atomic_helper.c b/drivers/gpu/drm/drm_atomic_helper.c
> > index 8be9719..19d0ea1 100644
> > --- a/drivers/gpu/drm/drm_atomic_helper.c
> > +++ b/drivers/gpu/drm/drm_atomic_helper.c
> > @@ -452,6 +452,69 @@ static int handle_conflicting_encoders(struct drm_atomic_state *state,
> >   	return 0;
> >   }
> > +static enum drm_mode_status mode_valid_path(struct drm_connector *connector,
> > +					    struct drm_encoder *encoder,
> > +					    struct drm_crtc *crtc,
> > +					    struct drm_display_mode *mode)
> > +{
> > +	enum drm_mode_status ret;
> > +
> > +	ret = drm_encoder_mode_valid(encoder, mode);
> > +	if (ret != MODE_OK) {
> > +		DRM_DEBUG_ATOMIC("[ENCODER:%d:%s] mode_valid() failed\n",
> > +				encoder->base.id, encoder->name);
> > +		return ret;
> > +	}
> > +
> > +	ret = drm_bridge_mode_valid(encoder->bridge, mode);
> > +	if (ret != MODE_OK) {
> > +		DRM_DEBUG_ATOMIC("[BRIDGE] mode_valid() failed\n");
> > +		return ret;
> > +	}
> > +
> > +	ret = drm_crtc_mode_valid(crtc, mode);
> > +	if (ret != MODE_OK) {
> > +		DRM_DEBUG_ATOMIC("[CRTC:%d:%s] mode_valid() failed\n",
> > +				crtc->base.id, crtc->name);
> > +		return ret;
> > +	}
> > +
> > +	return ret;
> > +}
> > +
> > +static int
> > +mode_valid(struct drm_atomic_state *state)
> > +{
> > +	struct drm_connector_state *conn_state;
> > +	struct drm_connector *connector;
> > +	int i;
> > +
> > +	for_each_new_connector_in_state(state, connector, conn_state, i) {
> > +		struct drm_encoder *encoder = conn_state->best_encoder;
> > +		struct drm_crtc *crtc = conn_state->crtc;
> > +		struct drm_crtc_state *crtc_state;
> > +		enum drm_mode_status mode_status;
> > +		struct drm_display_mode *mode;
> > +
> > +		if (!crtc || !encoder)
> > +			continue;
> > +
> > +		crtc_state = drm_atomic_get_new_crtc_state(state, crtc);
> > +		if (!crtc_state)
> > +			continue;
> > +		if (!crtc_state->mode_changed && !crtc_state->connectors_changed)
> > +			continue;
> > +
> > +		mode = &crtc_state->mode;
> > +
> > +		mode_status = mode_valid_path(connector, encoder, crtc, mode);
> > +		if (mode_status != MODE_OK)
> > +			return -EINVAL;
> > +	}
> > +
> > +	return 0;
> > +}
> > +
> >   /**
> >    * drm_atomic_helper_check_modeset - validate state object for modeset changes
> >    * @dev: DRM device
> > @@ -466,13 +529,15 @@ static int handle_conflicting_encoders(struct drm_atomic_state *state,
> >    * 2. &drm_connector_helper_funcs.atomic_check to validate the connector state.
> >    * 3. If it's determined a modeset is needed then all connectors on the affected crtc
> >    *    crtc are added and &drm_connector_helper_funcs.atomic_check is run on them.
> > - * 4. &drm_bridge_funcs.mode_fixup is called on all encoder bridges.
> > - * 5. &drm_encoder_helper_funcs.atomic_check is called to validate any encoder state.
> > + * 4. &drm_encoder_helper_funcs.mode_valid, &drm_bridge_funcs.mode_valid and
> > + *    &drm_crtc_helper_funcs.mode_valid are called on the affected components.
> > + * 5. &drm_bridge_funcs.mode_fixup is called on all encoder bridges.
> > + * 6. &drm_encoder_helper_funcs.atomic_check is called to validate any encoder state.
> >    *    This function is only called when the encoder will be part of a configured crtc,
> >    *    it must not be used for implementing connector property validation.
> >    *    If this function is NULL, &drm_atomic_encoder_helper_funcs.mode_fixup is called
> >    *    instead.
> > - * 6. &drm_crtc_helper_funcs.mode_fixup is called last, to fix up the mode with crtc constraints.
> > + * 7. &drm_crtc_helper_funcs.mode_fixup is called last, to fix up the mode with crtc constraints.
> >    *
> >    * &drm_crtc_state.mode_changed is set when the input mode is changed.
> >    * &drm_crtc_state.connectors_changed is set when a connector is added or
> > @@ -617,6 +682,10 @@ static int handle_conflicting_encoders(struct drm_atomic_state *state,
> >   			return ret;
> >   	}
> > +	ret = mode_valid(state);
> > +	if (ret)
> > +		return ret;
> > +
> 
> Since we've ensured that the modes won't fail, can mode_fixup() and
> the mode_fixup() ops for crtc/bridge/encoders be assured to not fail
> too? This is assuming that all drivers have moved to using the new
> mode_valid() ops correctly.

The entire point of re-checking is that userspace can create its own modes
and submit them to the kernel, bypassing the current
connector->mode_valid() check. I think almost all drivers get this wrong
unfortunately :(
-Daniel
-- 
Daniel Vetter
Software Engineer, Intel Corporation
http://blog.ffwll.ch

[toc] | [prev] | [standalone]


Back to top | Article view | linux.kernel


csiph-web